package de.ids_mannheim.korap.client;

import static org.mockserver.integration.ClientAndServer.startClientAndServer;
import static org.mockserver.model.HttpRequest.request;
import static org.mockserver.model.HttpResponse.response;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.content;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.header;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;

import org.junit.jupiter.api.AfterAll;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.mockserver.client.MockServerClient;
import org.mockserver.integration.ClientAndServer;
import org.mockserver.model.Header;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;

@SpringBootTest
@AutoConfigureMockMvc
class OAuth2ClientApplicationTests {

    @Value("${client.id}")
    private String clientId;
    @Value("${client.secret}")
    private String clientSecret;
    @Value("${client.redirect.uri}")
    private String redirectUri;

    @Value("${korap.authorize.url}")
    private String korapAuthorizeUrl;
    @Value("${korap.token.url}")
    private String korapTokenUrl;



    @Autowired
    private MockMvc mockMvc;

    private static ClientAndServer mockServer;
    private static MockServerClient mockClient;


    @BeforeAll
    public static void startMockServer () throws UnsupportedEncodingException {
        mockServer = startClientAndServer(1080);
        mockClient = new MockServerClient("localhost", mockServer.getPort());
    }


    @AfterAll
    public static void stopMockServer () {
        mockServer.stop();
    }

    @Test
    public void testRedirect () throws Exception {
        String encodedURL = URLEncoder.encode(redirectUri, "UTF-8");
        String locationUri = korapAuthorizeUrl + "/authorize?client_id="
                + clientId + "&redirect_uri=" + encodedURL
                + "&response_type=code&state=ZMwDGTZ2RY";

        mockMvc.perform(get("/authorize"))
                .andExpect(status().is3xxRedirection())
                .andExpect(header().string("location", locationUri));
    }


    @Test
    public void testAuthorization () throws Exception {
        String authorizationCode = "a8uZoar3t5zpif";
        String state = "9GKqaj4fX5cl";
        String oauth2Response = "{\"access_token\": \"4dcf8784ccfd26fac9bdb82778fe60e2\","
                + "\"refresh_token\" : \"hlWci75xb8atDiq3924NUSvOdtAh7Nlf9z\","
                + "\"token_type\": \"Bearer\","
                + "\"expires_in\": 259200,"
                + "\"state\":\""+state+"\"}";

        String tokenRequestParams = "client_id=" + clientId + "&client_secret="
                + clientSecret + "&grant_type=authorization_code&redirect_uri="
                + redirectUri + "&code=" + authorizationCode;

        // Simplified authorization server handling token requests
        mockClient.reset()
                .when(request()
                        .withMethod("POST")
                        .withPath("/token")
                        .withHeader(HttpHeaders.CONTENT_TYPE,
                                MediaType.APPLICATION_FORM_URLENCODED_VALUE)
                        .withBody(tokenRequestParams))
                .respond(response()
                        .withHeader(new Header("Content-Type",
                                "application/json; charset=utf-8"))
                        .withBody(oauth2Response).withStatusCode(200));

        

        // OAuth2 client receiving an authorization code and sending a token request        
        mockMvc.perform(get("/redirect")
                .param("code", authorizationCode)
                .param("state", state))
                .andExpect(status().isOk())
                .andExpect(content().json(oauth2Response));
    }
}
